@@ -93,7 +93,8 @@ class T5RelativePositionBias(nn.Module):
         ret += torch.where(is_small, n, val_if_large)
         return ret
 
-    def forward(self, i, j, *, device):
+    def forward(self, x):
+        i, j, device = *x.shape[-2:], x.device
         q_pos = torch.arange(i, dtype = torch.long, device = device)
         k_pos = torch.arange(j, dtype = torch.long, device = device)
         rel_pos = rearrange(k_pos, 'j -> 1 j') - rearrange(q_pos, 'i -> i 1')
@@ -208,18 +209,27 @@ class FLASH(nn.Module):
         causal = False,
         dropout = 0.,
         rotary_pos_emb = None,
-        norm_klass = nn.LayerNorm
+        norm_klass = nn.LayerNorm,
+        shift_tokens = False
     ):
         super().__init__()
         hidden_dim = int(dim * expansion_factor)
         self.group_size = group_size
         self.causal = causal
+        self.shift_tokens = shift_tokens
+
+        # positional embeddings
 
         self.rotary_pos_emb = rotary_pos_emb
+        self.rel_pos_bias = T5RelativePositionBias(query_key_dim ** 0.5, causal = causal)
+
+        # norm
 
         self.norm = norm_klass(dim)
         self.dropout = nn.Dropout(dropout)
 
+        # projections
+
         self.to_hidden = nn.Sequential(
             nn.Linear(dim, hidden_dim * 2),
             nn.SiLU()
@@ -231,15 +241,13 @@ class FLASH(nn.Module):
         )
 
         self.qk_offset_scale = OffsetScale(query_key_dim, heads = 4)
-
         self.to_out = nn.Linear(hidden_dim, dim)
 
     def forward(
         self,
         x,
         *,
-        mask = None,
-        rel_pos_bias = None
+        mask = None
     ):
         """
         b - batch
@@ -257,6 +265,13 @@ class FLASH(nn.Module):
 
         normed_x = self.norm(x)
 
+        # do token shift - a great, costless trick from an independent AI researcher in Shenzhen
+
+        if self.shift_tokens:
+            x_shift, x_pass = normed_x.chunk(2, dim = -1)
+            x_shift = F.pad(x_shift, (0, 0, 1, -1), value = 0.)
+            normed_x = torch.cat((x_shift, x_pass), dim = -1)
+
         # initial projections
 
         v, gate = self.to_hidden(normed_x).chunk(2, dim = -1)
@@ -269,7 +284,8 @@ class FLASH(nn.Module):
         # mask out linear attention keys
 
         if exists(mask):
-            lin_k = lin_k.masked_fill(~mask, 0.)
+            lin_mask = rearrange(mask, '... -> ... 1')
+            lin_k = lin_k.masked_fill(~lin_mask, 0.)
 
         # rotate queries and keys
 
@@ -297,8 +313,7 @@ class FLASH(nn.Module):
 
         sim = einsum('... i d, ... j d -> ... i j', quad_q, quad_k) / g
 
-        if exists(rel_pos_bias):
-            sim = sim + rel_pos_bias
+        sim = sim + self.rel_pos_bias(sim)
 
         attn = F.relu(sim) ** 2
         attn = self.dropout(attn)
@@ -324,8 +339,8 @@ class FLASH(nn.Module):
 
             lin_out = einsum('b g d e, b g n d -> b g n e', lin_kv, lin_q)
         else:
-            lin_kv = einsum('b g n d, b g n e -> b d e', lin_k, v) / n
-            lin_out = einsum('b g n d, b d e -> b g n e', lin_q, lin_kv)
+            lin_kv = einsum('b g n d, b g n e -> b g d e', lin_k, v) / n
+            lin_out = einsum('b g n d, b g d e -> b g n e', lin_q, lin_kv)
 
         # fold back groups into full sequence, and excise out padding
 
@@ -353,7 +368,8 @@ class FLASHTransformer(nn.Module):
         expansion_factor = 2.,
         causal = False,
         attn_dropout = 0.,
-        norm_type = 'scalenorm'
+        norm_type = 'scalenorm',
+        shift_tokens = True
     ):
         super().__init__()
         assert norm_type in ('scalenorm', 'layernorm'), 'norm_type must be one of scalenorm or layernorm'
@@ -365,12 +381,12 @@ class FLASHTransformer(nn.Module):
 
         self.token_emb = nn.Embedding(num_tokens, dim)
         self.abs_pos_emb = ScaledSinuEmbedding(dim)
-
         self.group_size = group_size
-        self.rel_pos_bias = T5RelativePositionBias(query_key_dim ** 0.5, causal = causal)
 
         rotary_pos_emb = RotaryEmbedding(dim = min(32, query_key_dim))
-        self.layers = nn.ModuleList([FLASH(dim = dim, group_size = group_size, query_key_dim = query_key_dim, expansion_factor = expansion_factor, causal = causal, dropout = attn_dropout, rotary_pos_emb = rotary_pos_emb, norm_klass = norm_klass) for _ in range(depth)])
+        # max rotary embedding dimensions of 32, partial Rotary embeddings, from Wang et al - GPT-J
+
+        self.layers = nn.ModuleList([FLASH(dim = dim, group_size = group_size, query_key_dim = query_key_dim, expansion_factor = expansion_factor, causal = causal, dropout = attn_dropout, rotary_pos_emb = rotary_pos_emb, norm_klass = norm_klass, shift_tokens = shift_tokens) for _ in range(depth)])
 
         self.to_logits = nn.Sequential(
             nn.LayerNorm(dim),
@@ -383,14 +399,10 @@ class FLASHTransformer(nn.Module):
         *,
         mask = None
     ):
-        device = x.device
-
         x = self.token_emb(x)
         x = self.abs_pos_emb(x) + x
 
-        rel_pos_bias = self.rel_pos_bias(self.group_size, self.group_size, device = device)
-
         for flash in self.layers:
-            x = flash(x, mask = mask, rel_pos_bias = rel_pos_bias)
+            x = flash(x, mask = mask)
 
         return self.to_logits(x)
